@@ -3,6 +3,14 @@
 
 ### ------- Part III --  "Matrix" (classes) specific ----------------------
 
+## lower.tri() and upper.tri()  -- masking  base definitions
+##	R/src/library/base/R/lower.tri.R
+##	R/src/library/base/R/upper.tri.R
+## but we do __not__ want to coerce to "base R" 'matrix' via as.matrix():
+##
+lower.tri <- function(x, diag = FALSE) if(diag) row(x) >= col(x) else row(x) > col(x)
+upper.tri <- function(x, diag = FALSE) if(diag) row(x) <= col(x) else row(x) < col(x)
+
 lsM <- function(...) {
     for(n in ls(..., envir=parent.frame()))
         if(is((. <- get(n)),"Matrix"))
@@ -18,27 +26,30 @@ asD <- function(m) { ## as "Dense"
 }
 
 ## "normal" sparse Matrix: Csparse, no diag="U"
-asCsp <- function(x) Matrix:::diagU2N(as(x, "CsparseMatrix"))
+asCsp <- function(x) diagU2N(as(x, "CsparseMatrix"))
 
+##' @title quasi-identical dimnames
 Qidentical.DN <- function(dx, dy) {
-    ## quasi-identical dimnames
+
     stopifnot(is.list(dx) || is.null(dx),
 	      is.list(dy) || is.null(dy))
     ## "empty"
     (is.null.DN(dx) && is.null.DN(dy)) || identical(dx, dy)
 }
 
+##' quasi-identical()  for 'Matrix' matrices
 Qidentical <- function(x,y, strictClass = TRUE) {
-    ## quasi-identical - for 'Matrix' matrices
-    if(class(x) != class(y)) {
-        if(strictClass || !is(x, class(y)))
+    if(!identical(class(x), cy <- class(y))) {
+        if(strictClass || !is(x, cy))
            return(FALSE)
         ## else try further
     }
     slts <- slotNames(x)
     if("Dimnames" %in% slts) { ## always (or we have no 'Matrix')
 	slts <- slts[slts != "Dimnames"]
-	if(!(Qidentical.DN(x@Dimnames, y@Dimnames)))
+	if(!Qidentical.DN(x@Dimnames, y@Dimnames) &&
+	   ## allow for "completion" of (NULL, <names>) dimnames of symmetricMatrix:
+	   !Qidentical.DN(dimnames(x), dimnames(y)))
 	    return(FALSE)
     }
     if("factors" %in% slts) { ## allow one empty and one non-empty 'factors'
@@ -53,6 +64,18 @@ Qidentical <- function(x,y, strictClass = TRUE) {
     TRUE
 }
 
+##' quasi-identical()  for traditional ('matrix') matrices
+mQidentical <- function(x,y, strictClass = TRUE) {
+    if(!identical(class(x), cy <- class(y))) {
+        if(strictClass || !is(x, cy))
+            return(FALSE)
+        ## else try further
+    }
+    if(!Qidentical.DN(dimnames(x), dimnames(y)))
+        return(FALSE)
+    identical(unname(x), unname(y))
+}
+
 Q.C.identical <- function(x,y, sparse = is(x,"sparseMatrix"),
                           checkClass = TRUE, strictClass = TRUE) {
     if(checkClass && class(x) != class(y)) {
@@ -71,7 +94,8 @@ Q.C.identical <- function(x,y, sparse = is(x,"sparseMatrix"),
 ##' @title  Quasi-equal for  'Matrix' matrices
 ##' @param x  Matrix
 ##' @param y  Matrix
-##' @param superclasses   x and y must coincide in (not) extending these
+##' @param superclasses  x and y must coincide in (not) extending these; set to empty,
+##'  if no class/inheritance checks should happen.
 ##' @param dimnames.check  logical indicating if dimnames(.) much match
 ##' @param tol  NA (--> use "==") or numerical tolerance for all.equal()
 ##' @return   logical:  Are x and y (quasi) equal ?
@@ -101,11 +125,11 @@ Q.eq <- function(x, y,
 	if(isDense)
 	    all(x == y | (is.na(x) & is.na(y)))
 	else ## 'x == y' blows up for large sparse matrices:
-	    isTRUE(all.equal(asC(x), asC(y), tol = 0.,
+	    isTRUE(all.equal(asC(x), asC(y), tolerance = 0.,
 			     check.attributes = dimnames.check))
     }
     else if(is.numeric(tol) && tol >= 0) {
-        isTRUE(all.equal(asC(x), asC(y), tol = tol,
+        isTRUE(all.equal(asC(x), asC(y), tolerance = tol,
                          check.attributes = dimnames.check))
     }
     else stop("'tol' must be NA or non-negative number")
@@ -158,11 +182,12 @@ sampleL <- function(n, size) {
 ##' @param m number of columns; default (=n)  ==> square matrix
 ##' @param density the desired sparseness density:
 ##' @param nnz number of non-zero entries; default from \code{density}
-##' @param giveCsparse logical specifying if result should be CsparseMatrix
-##' @return a [TC]sparseMatrix,  n x m
-##' @author Martin Maechler, Mar 2008
+##' @param repr character string specifying the sparseness kind of the result.
+##' @param giveCsparse *deprecated* logical specifying if result should be CsparseMatrix
+##' @return a [CTR]sparseMatrix,  n x m
+##' @author Martin Maechler, Mar 2008; July 2020 ('repr' instead og 'giveCsparse')
 rspMat <- function(n, m = n, density = 1/4, nnz = round(density * n*m),
-                   giveCsparse = TRUE)
+                   repr = c("C","T","R"), giveCsparse)
 {
     stopifnot(length(n) == 1, n == as.integer(n),
               length(m) == 1, m == as.integer(m),
@@ -172,16 +197,24 @@ rspMat <- function(n, m = n, density = 1/4, nnz = round(density * n*m),
     in0 <- sampleL(N, nnz)
     x <- sparseVector(i = in0, x = as.numeric(1L + seq_along(in0)), length = N)
     dim(x) <- c(n,m)#-> sparseMatrix
-    if (giveCsparse) as(x, "CsparseMatrix") else x
+    ## silent, back compatible (not yet warning about 'giveCsparse' deprecation):
+    repr <- if(missing(repr) && !missing(giveCsparse))
+		if(giveCsparse) "C" else "T"
+	    else match.arg(repr)
+    switch(repr,
+	   "C" = as(x, "CsparseMatrix"),
+	   "T" =    x,# TsparseMatrix
+	   "R" = as(x, "RsparseMatrix"))
 }
 
-
-## From \examples{..}  in ../man/sparseMatrix.Rd :
+## __DEPRECATED__ !!
 rSparseMatrix <- function(nrow, ncol, nnz,
 			  rand.x = function(n) round(rnorm(nnz), 2), ...)
 {
     stopifnot((nnz <- as.integer(nnz)) >= 0,
 	      nrow >= 0, ncol >= 0, nnz <= nrow * ncol)
+    .Deprecated("rsparsematrix")
+    ##=========
     sparseMatrix(i = sample(nrow, nnz, replace = TRUE),
 		 j = sample(ncol, nnz, replace = TRUE),
 		 x = rand.x(nnz), dims = c(nrow, ncol), ...)
@@ -206,26 +239,51 @@ rUnitTri <- function(n, upper = TRUE, ...)
     r
 }
 
-mkLDL <- function(n, density = 1/3) {
-    ## Purpose: make nice artificial   A = L D L'  (with exact numbers) decomp
-    ## ----------------------------------------------------------------------
-    ## Author: Martin Maechler, Date: 15 Mar 2008
-    stopifnot(n == round(n))
+##' Construct a nice (with exact numbers) random artificial  \eqn{A = L D L'}
+##' decomposition with a sparse \eqn{n \times n}{n x n} matrix \code{A} of
+##' density \code{density} and square root \eqn{D} determined by \code{d0}.
+##'
+##' If one of \code{rcond} or \code{condest} is true, \code{A} must be
+##' non-singular, both use an \eqn{LU} decomposition requiring
+##' non-singularity.
+##' @title Make Nice Artificial   A = L D L'  (With Exact Numbers) Decomposition
+##' @param n matrix dimension \eqn{n \times n}{n x n}
+##' @param density ratio of number of non-zero entries to total number
+##' @param d0 The sqrt of the diagonal entries of D default \code{10}, to be
+##' \dQuote{different} from \code{L} entries.
+##' @param rcond logical indicating if \code{\link{rcond}(A, useInv=TRUE)}
+##' should be returned which requires non-singular A and D.
+##' @param condest logical indicating if \code{\link{condest}(A)$est}
+##' should be returned which requires non-singular A and D.
+##' @return list with entries A, L, d.half, D, ..., where A inherits from
+##' class \code{"\linkS4class{symmetricMatrix}"} and should be equal to
+##' \code{as(L \%*\% D \%*\% t(L), "symmetricMatrix")}.
+##' @author Martin Maechler, Date: 15 Mar 2008
+mkLDL <- function(n, density = 1/3,
+                  d0 = 10, d.half = d0 * sample.int(n), # random permutation
+                  rcond = (n < 99), condest = (n >= 100))
+{
+    stopifnot(n == round(n), density <= 1)
     n <- as.integer(n)
+    stopifnot(n >= 1, is.numeric(d.half),
+              length(d.half) == n, d.half >= 0)
     L <- Matrix(0, n,n)
-
-    nnz <- round(n*n * density)
+    nnz <- round(density * n*n)
     L[sample(n*n, nnz)] <- seq_len(nnz)
-    L <- tril(L,-1)
+    L <- tril(L, -1L)
     diag(L) <- 1
-    d.half <- sample(10*(n:1))# random permutation ; use '10*' to be "different" from L entries
-    D <- Diagonal(x = d.half * d.half)
+    dh2 <- d.half^2
+    non.sing <- sum(dh2 > 0) == n
+    D <- Diagonal(x = dh2)
     A <- tcrossprod(L * rep(d.half, each=n))
     ## = as(L %*% D %*% t(L), "symmetricMatrix")
-    list(A = A, L = L, d.half = d.half, D = D)
+    list(A = A, L = L, d.half = d.half, D = D,
+	 rcond.A = if (rcond  && non.sing) rcond(A, useInv=TRUE),
+	 cond.A  = if(condest && non.sing) condest(A)$est)
 }
 
-eqDeterminant <- function(m1, m2, NA.Inf.ok=FALSE, ...) {
+eqDeterminant <- function(m1, m2, NA.Inf.ok=FALSE, tol=.Machine$double.eps^0.5, ...)
+{
     d1 <- determinant(m1) ## logarithm = TRUE
     d2 <- determinant(m2)
     d1m <- as.vector(d1$modulus)# dropping attribute
@@ -243,7 +301,7 @@ eqDeterminant <- function(m1, m2, NA.Inf.ok=FALSE, ...) {
     if(is.infinite(d1m)) d1$modulus <- sign(d1m)* .Machine$double.xmax
     if(is.infinite(d2m)) d2$modulus <- sign(d2m)* .Machine$double.xmax
     ## now they are finite or *one* of them is NA/NaN, and all.equal() will tell so:
-    all.equal(d1, d2, ...)
+    all.equal(d1, d2, tolerance=tol, ...)
 }
 
 ##' @param A a non-negative definite sparseMatrix, typically "dsCMatrix"
@@ -302,14 +360,18 @@ allCholesky <- function(A, verbose = FALSE, silentTry = FALSE)
 	 r.uniq = CHM_to_pLs(r1[ ! dup.r1]))
 }
 
+##' Cheap  Boolean Arithmetic Matrix product
+##' Should be equivalent to  %&%  which is faster [not for large dense!].
+##' Consequently mainly used in  checkMatrix()
+boolProd <- function(x,y) as((abs(x) %*% abs(y)) > 0, "nMatrix")
 
 ###----- Checking a "Matrix" -----------------------------------------
 
-##' <description>
+##' Check the compatibility of \pkg{Matrix} package Matrix with a
+##' \dQuote{traditional} \R matrix and perform a host of internal consistency
+##' checks.
 ##'
-##' <details>
-##' @title Compatibility tests "Matrix" <-> "traditional matrix"
-##'        and many more consistency checks
+##' @title Check Compatibility of Matrix Package Matrix with Traditional R Matrices
 ##'
 ##' @param m   a "Matrix"
 ##' @param m.m as(m, "matrix")  {if 'do.matrix' }
@@ -330,8 +392,8 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 			do.matrix = !isSparse || prod(dim(m)) < 1e6,
 			do.t = TRUE, doNorm = TRUE, doOps = TRUE,
                         doSummary = TRUE, doCoerce = TRUE,
-			doCoerce2 = doCoerce && !extends(cld, "RsparseMatrix"),
-			do.prod = do.t && do.matrix && !extends(cld, "RsparseMatrix"),
+			doCoerce2 = doCoerce && !isRsp, doDet = do.matrix,
+			do.prod = do.t && do.matrix && !isRsp,
 			verbose = TRUE, catFUN = cat)
 {
     ## is also called from  dotestMat()  in ../tests/Class+Meth.R
@@ -342,13 +404,16 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
     clNam <- class(m)
     cld <- getClassDef(clNam) ## extends(cld, FOO) is faster than is(m, FOO)
     isCor    <- extends(cld, "corMatrix")
-    isSparse <- extends(cld, "sparseMatrix") # also true for diagonalMatrix !
     isSym    <- extends(cld, "symmetricMatrix")
-    isDiag   <- extends(cld, "diagonalMatrix")
-    isPerm   <- extends(cld, "pMatrix")
-    isTri <- !isSym && !isDiag && !isPerm && extends(cld, "triangularMatrix")
+    if(isSparse <- extends(cld, "sparseMatrix")) { # also true for these
+	isRsp  <- extends(cld, "RsparseMatrix")
+	isDiag <- extends(cld, "diagonalMatrix")
+	isInd  <- extends(cld, "indMatrix")
+	isPerm <- extends(cld, "pMatrix")
+    } else isRsp <- isDiag <- isInd <- isPerm <- FALSE
+    isTri <- !isSym && !isDiag && !isInd && extends(cld, "triangularMatrix")
     is.n     <- extends(cld, "nMatrix")
-    nonMatr  <- clNam != Matrix:::MatrixClass(clNam, cld)
+    nonMatr  <- clNam != (Mcl <- MatrixClass(clNam, cld))
 
     Cat	 <- function(...) if(verbose) cat(...)
     CatF <- function(...) if(verbose) catFUN(...)
@@ -384,13 +449,17 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	if(isSym) ## check that t() swaps 'uplo'  L <--> U :
 	    stopifnot(c("L","U") == sort(c(m@uplo, tm@uplo)))
 	ttm <- t(tm)
-	if(extends(cld, "CsparseMatrix") ||
-	   extends(cld, "generalMatrix") || isDiag)
+        ## notInd: "pMatrix" ok, but others inheriting from "indMatrix" are not
+        notInd <- (!isInd || isPerm)
+	if(notInd && (extends(cld, "CsparseMatrix") ||
+	   extends(cld, "generalMatrix") || isDiag))
             stopifnot(Qidentical(m, ttm, strictClass = !nonMatr))
-	else if(do.matrix)
-	    stopifnot(nonMatr || class(ttm) == clNam,
-                      all(m == ttm | ina))
-        ## else : not testing
+	else if(do.matrix) {
+	    if(notInd) stopifnot(nonMatr || class(ttm) == clNam)
+	    stopifnot(all(m == ttm | ina))
+	    ## else : not testing
+	}
+
 
 	## crossprod()	%*%  etc
 	if(do.prod) {
@@ -401,7 +470,12 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 		      dim(tcm) == rep.int(nrow(m), 2),
 		      ## FIXME: %*% drops dimnames
 		      Q.eq2(c.m, tm %*% m, tol = tolQ),
-		      Q.eq2(tcm, m %*% tm, tol = tolQ))
+		      Q.eq2(tcm, m %*% tm, tol = tolQ),
+                      ## should work with dimnames:
+		      Q.eq(m %&% tm, boolProd(m, tm), superclasses=NULL, tol = 0)
+                     ,
+		      Q.eq(tm %&% m, boolProd(tm, m), superclasses=NULL, tol = 0)
+                      )
 	}
     }
     if(!do.matrix) {
@@ -444,7 +518,7 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	stopifnot(is(vm, "Matrix"), validObject(vm), dim(vm) == c(d[1]*d[2], 1))
     }
 
-    if(!isPerm)
+    if(!isInd)
         m.d <- local({ m. <- m; diag(m.) <- diag(m); m. })
     if(do.matrix)
     stopifnot(identical(dim(m.m), dim(m)),
@@ -463,10 +537,10 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
     if(isSparse) {
 	n0m <- drop0(m) #==> n0m is Csparse
 	has0 <- !Qidentical(n0m, as(m,"CsparseMatrix"))
-	if(!isPerm && !extends(cld, "RsparseMatrix") &&
-           !(extends(cld, "TsparseMatrix") && Matrix:::is_duplicatedT(m, di = d)))
+	if(!isInd && !isRsp &&
+           !(extends(cld, "TsparseMatrix") && anyDuplicatedT(m, di = d)))
                                         # 'diag<-' is does not change attrib:
-	    stopifnot(identical(m, m.d))
+	    stopifnot(Qidentical(m, m.d))# e.g., @factors may differ
     }
     else if(!identical(m, m.d)) { # dense : 'diag<-' is does not change attrib
 	if(isTri && m@diag == "U" && m.d@diag == "N" &&
@@ -493,7 +567,7 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	ns <- as(m, "nsparseMatrix")
 	stopifnot(identical(n1,ns),
 		  isDiag || ((if(isSym) Matrix:::nnzSparse else sum)(n1) ==
-			     length(if(isPerm) m@perm else Matrix:::diagU2N(m)@x)))
+			     length(if(isInd) m@perm else diagU2N(m)@x)))
         Cat("ok\n")
     }
 
@@ -533,8 +607,10 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 		CatF("symmpart(m) + skewpart(m) == m: ")
 		Q.eq.symmpart(m)
 		CatF("ok;  determinant(): ")
-		if(any(is.na(m.m)) && extends(cld, "triangularMatrix"))
-		    Cat(" skipped: is triang. and has NA")
+		if(!doDet)
+		    Cat(" skipped (!doDet): ")
+		else if(any(is.na(m.m)) && extends(cld, "triangularMatrix"))
+		    Cat(" skipped: is triang. and has NA: ")
 		else
 		    stopifnot(eqDeterminant(m, m.m, NA.Inf.ok=TRUE))
 		Cat("ok\n")
@@ -550,16 +626,16 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
     }
 
     if(doCoerce2 && do.matrix) { ## not for large m:  !m will be dense
-
 	if(is.n) {
-	    stopifnot(identical(m, as(as(m, "dMatrix"),"nMatrix")),
-		      identical(m, as(as(m, "lMatrix"),"nMatrix")),
+	    mM <- if(nonMatr) as(m, Mcl) else m
+	    stopifnot(identical(mM, as(as(m, "dMatrix"),"nMatrix")),
+		      identical(mM, as(as(m, "lMatrix"),"nMatrix")),
 		      identical(which(m), which(m.m)))
 	}
 	else if(extends(cld, "lMatrix")) { ## should fulfill even with NA:
 	    stopifnot(all(m | !m | ina), !any(!m & m & !ina))
 	    if(extends(cld, "TsparseMatrix")) # allow modify, since at end here
-		m <- Matrix:::uniqTsparse(m, clNam)
+		m <- uniqTsparse(m, clNam)
 	    stopifnot(identical(m, m & TRUE),
 		      identical(m, FALSE | m))
 	    ## also check the  coercions to [dln]Matrix
@@ -599,7 +675,7 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	    Cat("valid:", validObject(tm), "\n")
 	    if(m@uplo == tm@uplo) ## otherwise, the matrix effectively was *diagonal*
 		## note that diagU2N(<dtr>) |-> dtC :
-		stopifnot(Qidentical(tm, as(Matrix:::diagU2N(m), clNam)))
+		stopifnot(Qidentical(tm, as(diagU2N(m), clNam)))
 	}
 	else if(isDiag) {
 
@@ -620,3 +696,50 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 
     invisible(TRUE)
 }
+
+### --- These use
+
+##' Check QR-consistency of dense and sparse
+chk.qr.D.S <- function(d., s., y, Y = Matrix(y), force = FALSE, tol = 1e-10) {
+    stopifnot(is.qr(d.), is(s., "sparseQR"))
+    cc <- qr.coef(d.,y)
+    rank.def <- any(is.na(cc)) && d.$rank < length(d.$pivot)
+    if(rank.def && force) cc <- mkNA.0(cc) ## set NA's to 0 .. ok, in some case
+
+    ## when system is rank deficient, have differing cases, not always just NA <-> 0 coef
+    ## FIXME though:  resid & fitted should be well determined
+    if(force || !rank.def) stopifnot(
+### FIXME: temporary:
+###	is.all.equal3(	    cc	     , drop(qr.coef  (s.,y)), drop(qr.coef  (s.,Y)), tol=tol),
+	is.all.equal3(	unname( cc ) , drop(qr.coef  (s.,y)), drop(qr.coef  (s.,Y)), tol=tol),
+### END{FIXME}
+	is.all.equal3(qr.resid (d.,y), drop(qr.resid (s.,y)), drop(qr.resid (s.,Y)), tol=tol),
+	is.all.equal3(qr.fitted(d.,y), drop(qr.fitted(s.,y)), drop(qr.fitted(s.,Y)), tol=tol)
+	)
+}
+
+##' "Combi" calling chkQR() on both "(sparse)Matrix" and 'traditional' version
+##' ------  and combine the two qr decompositions using chk.qr.D.S()
+##'         [ chkQR() def. in >>>>> ./test-tools-1.R <<<<< ]
+##'
+##' @title check QR-decomposition, and compare sparse and dense one
+##' @param A a 'Matrix' , typically 'sparseMatrix'
+##' @param Qinv.chk
+##' @param QtQ.chk
+##' @param quiet
+##' @return list with 'qA' (sparse QR) and 'qa' (traditional (dense) QR)
+##' @author Martin Maechler
+checkQR.DS.both <- function(A, Qinv.chk, QtQ.chk=NA,
+                            quiet=FALSE, giveRE=TRUE, tol = 1e-13)
+{
+    stopifnot(is(A,"Matrix"))
+    if(!quiet) cat("classical: ")
+    qa <- chkQR(as(A, "matrix"), Qinv.chk=TRUE, QtQ.chk=TRUE, tol=tol, giveRE=giveRE)# works always
+    if(!quiet) cat("[Ok] ---  sparse: ")
+    qA <- chkQR(A, Qinv.chk=Qinv.chk, QtQ.chk=QtQ.chk, tol=tol, giveRE=giveRE)
+    validObject(qA)
+    if(!quiet) cat("[Ok]\n")
+    chk.qr.D.S(qa, qA, y = 10 + 1:nrow(A), tol = 256*tol)# ok [not done in rank deficient case!]
+    invisible(list(qA=qA, qa=qa))
+}
+
