@@ -1,7 +1,7 @@
 #  File src/library/tools/R/sotools.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2011-2020 The R Core Team
+#  Copyright (C) 2011-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -56,18 +56,25 @@ if(.Platform$OS.type == "windows") {
 
 read_symbols_from_object_file <- function(f)
 {
-    ## reasonable to assume this on the path
-    if(!nzchar(nm <- Sys.which("nm"))) {
+    ## For GCC & LTO, we need a different command, possibly with args
+    ## On macOS, the system nm works with LTO objects.
+    ## Do not use NM as make sets it.
+    nm <- Sys.getenv("UserNM")
+    if(!nzchar(nm)) {
+        ## reasonable to assume nm is on the path
+        nm <- Sys.which("nm")
+        if(!nzchar(nm)) nm <- shQuote(nm)
+    }
+    if(!nzchar(nm)) {
         warning("this requires 'nm' to be on the PATH")
         return()
     }
     f <- file_path_as_absolute(f)
     if(!(file.size(f))) return()
-    s <- strsplit(system(sprintf("%s -Pg %s", shQuote(nm), shQuote(f)),
+    s <- strsplit(system(sprintf("%s -Pg %s", nm, shQuote(f)),
                          intern = TRUE),
                   " +")
-    ## Cannot simply rbind() this because elements may have 2-4
-    ## entries.
+    ## Cannot simply rbind() this because elements may have 2-4 entries.
     n <- length(s)
     tab <- matrix("", nrow = n, ncol = 4L)
     colnames(tab) <- c("name", "type", "value", "size")
@@ -95,6 +102,8 @@ get_system_ABI <- if(.Platform$OS.type == "windows") {
 
 system_ABI <- get_system_ABI()
 
+## entry points for std::terminate are commented out as almost all
+## come from system headers.
 so_symbol_names_table <-
     ## 'linux' == glibc, principally
     c("linux, C, gcc, abort, abort",
@@ -115,6 +124,7 @@ so_symbol_names_table <-
       "linux, C, gcc, vprintf, __vprintf_chk",
       "linux, C++, gxx, std::cout, _ZSt4cout",
       "linux, C++, gxx, std::cerr, _ZSt4cerr",
+      #"linux, C++, gxx, std::terminate, _ZSt9terminatev",
       "linux, C, gcc, rand, rand",
       "linux, C, gcc, random, random",
       "linux, C, gcc, rand_r, rand_r",
@@ -146,38 +156,40 @@ so_symbol_names_table <-
       "linux, Fortran, flang, stop, f90_stop08",
       "linux, Fortran, flang, rand, rand",
 
-      "osx, C, gcc, abort, _abort",
-      "osx, C, gcc, assert, ___assert_rtn",
-      "osx, C, gcc, exit, _exit",
-      "osx, C, gcc, _exit, __exit",
-      "osx, C, gcc, _Exit, __Exit",
-      "osx, C, gcc, printf, _printf",
-      "osx, C, gcc, printf, _puts",
-      "osx, C, gcc, puts, _puts",
-      "osx, C, gcc, putchar, _putchar",
-      "osx, C, gcc, stderr, ___stderrp",
-      "osx, C, gcc, stdout, ___stdoutp",
-      "osx, C, gcc, vprintf, _vprintf",
-      "osx, C++, gxx, std::cout, __ZSt4cout",
-      "osx, C++, gxx, std::cerr, __ZSt4cerr",
-      "osx, C, gcc, rand, _rand",
-      "osx, C, gcc, random, _random",
-      "osx, C, gcc, rand_r, _rand_r",
-      "osx, C, gcc, srand, _srand",
-      "osx, C, gcc, srandom, _srandom",
-      "osx, C, gcc, srand48, _srand48",
+      ## clang identifies itself as gcc, so configure has used that
+      "macos, C, gcc, abort, _abort",
+      "macos, C, gcc, assert, ___assert_rtn",
+      "macos, C, gcc, exit, _exit",
+      "macos, C, gcc, _exit, __exit",
+      "macos, C, gcc, _Exit, __Exit",
+      "macos, C, gcc, printf, _printf",
+      "macos, C, gcc, printf, _puts",
+      "macos, C, gcc, puts, _puts",
+      "macos, C, gcc, putchar, _putchar",
+      "macos, C, gcc, stderr, ___stderrp",
+      "macos, C, gcc, stdout, ___stdoutp",
+      "macos, C, gcc, vprintf, _vprintf",
+      "macos, C++, gxx, std::cout, __ZSt4cout",
+      "macos, C++, gxx, std::cerr, __ZSt4cerr",
+      #"macos, C++, gxx, std::terminate, __ZSt9terminatev",
+      "macos, C, gcc, rand, _rand",
+      "macos, C, gcc, random, _random",
+      "macos, C, gcc, rand_r, _rand_r",
+      "macos, C, gcc, srand, _srand",
+      "macos, C, gcc, srandom, _srandom",
+      "macos, C, gcc, srand48, _srand48",
       ## libc++ variants
-      "osx, C++, gxx, std::cout, __ZNSt3__14coutE",
-      "osx, C++, gxx, std::cerr, __ZNSt3__14cerrE",
-      "osx, Fortran, gfortran, open, __gfortran_st_open",
-      "osx, Fortran, gfortran, close, __gfortran_st_close",
-      "osx, Fortran, gfortran, rewind, _gfortran_st_rewind",
-      "osx, Fortran, gfortran, read, __gfortran_st_read",
-      "osx, Fortran, gfortran, write, __gfortran_st_write",
-      "osx, Fortran, gfortran, print, __gfortran_st_write",
-      "osx, Fortran, gfortran, stop, __gfortran_stop_numeric",
-      "osx, Fortran, gfortran, stop, __gfortran_stop_string",
-      "osx, Fortran, gfortran, rand, __gfortran_rand",
+      "macos, C++, gxx, std::cout, __ZNSt3__14coutE",
+      "macos, C++, gxx, std::cerr, __ZNSt3__14cerrE",
+      "macos, Fortran, gfortran, open, __gfortran_st_open",
+      "macos, Fortran, gfortran, close, __gfortran_st_close",
+      "macos, Fortran, gfortran, rewind, _gfortran_st_rewind",
+      "macos, Fortran, gfortran, read, __gfortran_st_read",
+      "macos, Fortran, gfortran, write, __gfortran_st_write",
+      "macos, Fortran, gfortran, print, __gfortran_st_write",
+      "macos, Fortran, gfortran, stop, __gfortran_stop_numeric",
+      "macos, Fortran, gfortran, stop, __gfortran_stop_string",
+      "macos, Fortran, gfortran, rand, __gfortran_rand",
 
       "freebsd, C, gcc, abort, abort",
       "freebsd, C, gcc, assert, __assert",
@@ -220,6 +232,7 @@ so_symbol_names_table <-
       "solaris, C, solcc, vprintf, vprintf",
       "solaris, C++, solCC, std::cout, __1cDstdEcout_",
       "solaris, C++, solCC, std::cerr, __1cDstdEcerr_",
+      #"solaris, C++, solCC, std::terminate, _ZSt9terminatev",
       "solaris, C, solcc, random, random",
       "solaris, C, solcc, rand, rand",
       "solaris, C, solcc, rand_r, rand_r",
@@ -260,6 +273,8 @@ so_symbol_names_table <-
       "solaris, C, gcc, srand48, srand48",
       "solaris, C++, gxx, std::cout, _ZSt4cout",
       "solaris, C++, gxx, std::cerr, _ZSt4cerr",
+      "solaris, C++, gxx, std::cerr, _ZSt4cerr",
+      #"solaris, C++, gxx, std::terminate, _ZSt9terminatev",
       "solaris, Fortran, gfortran, open, _gfortran_st_open",
       "solaris, Fortran, gfortran, close, _gfortran_st_close",
       "solaris, Fortran, gfortran, rewind, _gfortran_st_rewind",
@@ -274,6 +289,7 @@ so_symbol_names_table <-
       ## only in .o, positions hard-coded in check_so_symbols
       "windows, C++, g++, std::cout, _ZSt4cout",
       "windows, C++, g++, std::cerr, _ZSt4cerr",
+      #"windows, C++, gxx, std::terminate, _ZSt9terminatev",
       "windows, Fortran, gfortran, open, _gfortran_st_open",
       "windows, Fortran, gfortran, close, _gfortran_st_close",
       "windows, Fortran, gfortran, rewind, _gfortran_st_rewind",
@@ -403,8 +419,9 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
             "locale2charset", "match5", "matherr",
             "max_contour_segments", "mbcsToUcs2", "memtrace_report",
             "parseError", "pythag_", "rs_", "rwarnc_",
-            "signrank_free", "tql2_", "tqlrat_", "tred1_", "tred2_",
-            "utf8locale", "wilcox_free", "yylloc",
+            "tql2_", "tqlrat_", "tred1_", "tred2_", "utf8locale", "yylloc",
+            ## "signrank_free", "wilcox_free" are API only from 4.2.0
+            ## but WRE advised using them long before.
 
 ## Rinterface.h, Rembedded.h, R_ext/{RStartup,eventloop}.h
             "AllDevicesKilled", "R_CStackLimit", "R_CStackStart",
