@@ -3,7 +3,7 @@
  *  file selectlist.c
  *  Copyright (C) 1998--2003  Guido Masarotto and Brian Ripley
  *  Copyright (C) 2004	      The R Foundation
- *  Copyright (C) 2005--2013  The R Core Team
+ *  Copyright (C) 2005--2020  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +34,6 @@
 static window wselect;
 static button bFinish, bCancel;
 static listbox f_list;
-static char selected[100];
 static int done;
 
 static void cleanup(void)
@@ -47,13 +46,11 @@ static void cleanup(void)
 
 static void cancel(button b)
 {
-    strcpy(selected, "");
     done = 2;
 }
 
 static void finish(button b)
 {
-    strncpy(selected, GA_gettext(f_list), 100);
     done = 1;
 }
 
@@ -141,8 +138,17 @@ SEXP Win_selectlist(SEXP args)
 	} else { /* cancel */
 	    PROTECT(ans = allocVector(STRSXP, 0));
 	}
-    } else
-	PROTECT(ans = mkString(selected));
+    } else {
+	if (done == 1) { /* Finish */
+	    i = getlistitem(f_list);
+	    if (i >= 0) 
+		PROTECT(ans = mkString(clist[i]));
+	    else 
+		PROTECT(ans = mkString("")); /* error/unreachable */
+	} else { /* cancel */
+	    PROTECT(ans = mkString(""));
+	}
+    }
 
     cleanup();
     show(RConsole);
@@ -163,8 +169,8 @@ static int countFilenamesW(const wchar_t *list)
 
 static SEXP mkCharUTF8(const wchar_t *wc)
 {
-    char s[4*MAX_PATH];
-    wcstoutf8(s, wc, 4*MAX_PATH);
+    char s[4*MAX_PATH + 1];
+    wcstoutf8(s, wc, sizeof(s));
     return mkCharCE(s, CE_UTF8);
 }
 
@@ -207,8 +213,11 @@ SEXP chooseFiles(SEXP def, SEXP caption, SEXP smulti, SEXP filters, SEXP sindex)
     res = askfilenamesW(filenameToWchar(STRING_ELT(caption, 0), 0), path,
 			multi, cfilters, filterindex, NULL);
 
-    count = countFilenamesW(res);
-
+    if (multi)
+    	count = countFilenamesW(res);
+    else
+    	count = wcslen(res) ? 1 : 0;
+    	
     SEXP ans;
     if (count < 2) PROTECT(ans = allocVector(STRSXP, count));
     else PROTECT(ans = allocVector(STRSXP, count-1));
